@@ -134,16 +134,19 @@ extern int h_errno;
 
 /*** Ping packet defines ***/
 
-#define MAX_IP_PACKET 65536 /* (theoretical) max IP packet size */
-#define SIZE_IP_HDR 40
+#define MAX_IP_PACKET 65535 /* (theoretical) max IPv4 packet size */
+#define SIZE_IP_HDR 20 /* min IPv4 header size */
 #define SIZE_ICMP_HDR 8 /* from ip_icmp.h */
 #define MAX_PING_DATA (MAX_IP_PACKET - SIZE_IP_HDR - SIZE_ICMP_HDR)
 
-#define MAX_GENERATE 100000 /* maximum number of hosts that -g can generate */
+#define MAX_GENERATE 131070 /* maximum number of hosts that -g can generate */
 
 /* sized so as to be like traditional ping */
 #define DEFAULT_PING_DATA_SIZE 56
 
+/* ICMP Timestamp has a fixed payload size of 12 bytes */
+#define ICMP_TIMESTAMP_DATA_SIZE 12
+
 /* maxima and minima */
 #ifdef FPING_SAFE_LIMITS
 #define MIN_INTERVAL 1 /* in millisec */
@@ -357,7 +360,14 @@ int per_recv_flag, report_all_rtts_flag, name_flag, addr_flag, backoff_flag, rdn
 int multif_flag, timeout_flag, fast_reachable;
 int outage_flag = 0;
 int timestamp_flag = 0;
+int timestamp_format_flag = 0;
 int random_data_flag = 0;
+int cumulative_stats_flag = 0;
+int check_source_flag = 0;
+int icmp_request_typ = 0;
+int print_tos_flag = 0;
+int print_ttl_flag = 0;
+int size_flag = 0;
 #if defined(DEBUG) || defined(_DEBUG)
 int randomly_lose_flag, trace_flag, print_per_system_flag;
 int lose_factor;
@@ -392,6 +402,7 @@ struct event *ev_dequeue(struct event_queue *queue);
 void ev_remove(struct event_queue *queue, struct event *event);
 void add_cidr(char *);
 void add_range(char *, char *);
+void add_addr_range_ipv4(unsigned long, unsigned long);
 void print_warning(char *fmt, ...);
 int addr_cmp(struct sockaddr *a, struct sockaddr *b);
 void host_add_ping_event(HOST_ENTRY *h, int index, int64_t ev_time);
@@ -399,6 +410,8 @@ void host_add_timeout_event(HOST_ENTRY *h, int index, int64_t ev_time);
 struct event *host_get_timeout_event(HOST_ENTRY *h, int index);
 void stats_add(HOST_ENTRY *h, int index, int success, int64_t latency);
 void update_current_time();
+void print_timestamp_format(int64_t current_time_ns, int timestamp_format);
+static uint32_t ms_since_midnight_utc(int64_t time_val);
 
 /************************************************************
 
@@ -521,6 +534,7 @@ int main(int argc, char **argv)
         { "vcount", 'C', OPTPARSE_REQUIRED },
         { "rdns", 'd', OPTPARSE_NONE },
         { "timestamp", 'D', OPTPARSE_NONE },
+        { "timestamp-format", '0', OPTPARSE_REQUIRED },
         { "elapsed", 'e', OPTPARSE_NONE },
         { "file", 'f', OPTPARSE_REQUIRED },
         { "generate", 'g', OPTPARSE_NONE },
@@ -528,6 +542,7 @@ int main(int argc, char **argv)
         { "ttl", 'H', OPTPARSE_REQUIRED },
         { "interval", 'i', OPTPARSE_REQUIRED },
         { "iface", 'I', OPTPARSE_REQUIRED },
+        { "icmp-timestamp", '0', OPTPARSE_NONE },
 #ifdef SO_MARK
         { "fwmark", 'k', OPTPARSE_REQUIRED },
 #endif
@@ -551,6 +566,9 @@ int main(int argc, char **argv)
         { "version", 'v', OPTPARSE_NONE },
         { "reachable", 'x', OPTPARSE_REQUIRED },
         { "fast-reachable", 'X', OPTPARSE_REQUIRED },
+        { "check-source", '0', OPTPARSE_NONE },
+        { "print-tos", '0', OPTPARSE_NONE },
+        { "print-ttl", '0', OPTPARSE_NONE },
 #if defined(DEBUG) || defined(_DEBUG)
         { NULL, 'z', OPTPARSE_REQUIRED },
 #endif
@@ -560,6 +578,37 @@ int main(int argc, char **argv)
     float opt_value_float;
     while ((c = optparse_long(&optparse_state, longopts, NULL)) != EOF) {
         switch (c) {
+        case '0':
+            if(strstr(optparse_state.optlongname, "timestamp-format") != NULL) {
+                if(strcmp(optparse_state.optarg, "ctime") == 0) {
+                  timestamp_format_flag = 1;
+                }else if(strcmp(optparse_state.optarg, "iso") == 0) {
+                  timestamp_format_flag = 2;
+                }else if(strcmp(optparse_state.optarg, "rfc3339") == 0) {
+                  timestamp_format_flag = 3;
+                }else{
+                  usage(1);
+                }
+            } else if (strstr(optparse_state.optlongname, "check-source") != NULL) {
+                check_source_flag = 1;
+            } else if (strstr(optparse_state.optlongname, "icmp-timestamp") != NULL) {
+#ifdef IPV6
+                if (hints_ai_family != AF_UNSPEC && hints_ai_family != AF_INET) {
+                    fprintf(stderr, "%s: ICMP Timestamp is IPv4 only\n", prog);
+                    exit(1);
+                }
+                hints_ai_family = AF_INET;
+#endif
+                icmp_request_typ = 13;
+                ping_data_size = ICMP_TIMESTAMP_DATA_SIZE;
+            } else if (strstr(optparse_state.optlongname, "print-tos") != NULL) {
+                print_tos_flag = 1;
+            } else if (strstr(optparse_state.optlongname, "print-ttl") != NULL) {
+                print_ttl_flag = 1;
+            } else {
+                usage(1);
+            }
+            break;
         case '4':
 #ifdef IPV6
             if (hints_ai_family != AF_UNSPEC && hints_ai_family != AF_INET) {
@@ -604,7 +653,7 @@ int main(int argc, char **argv)
             break;
 
         case 't':
-            if (!sscanf(optparse_state.optarg, "%f", &opt_value_float))
+            if (sscanf(optparse_state.optarg, "%f", &opt_value_float) != 1)
                 usage(1);
             if (opt_value_float < 0) {
                 usage(1);
@@ -614,12 +663,12 @@ int main(int argc, char **argv)
             break;
 
         case 'r':
-            if (!sscanf(optparse_state.optarg, "%u", &retry))
+            if (sscanf(optparse_state.optarg, "%u", &retry) != 1)
                 usage(1);
             break;
 
         case 'i':
-            if (!sscanf(optparse_state.optarg, "%f", &opt_value_float))
+            if (sscanf(optparse_state.optarg, "%f", &opt_value_float) != 1)
                 usage(1);
             if (opt_value_float < 0) {
                 usage(1);
@@ -628,7 +677,7 @@ int main(int argc, char **argv)
             break;
 
         case 'p':
-            if (!sscanf(optparse_state.optarg, "%f", &opt_value_float))
+            if (sscanf(optparse_state.optarg, "%f", &opt_value_float) != 1)
                 usage(1);
             if (opt_value_float < 0) {
                 usage(1);
@@ -653,9 +702,9 @@ int main(int argc, char **argv)
             break;
 
         case 'b':
-            if (!sscanf(optparse_state.optarg, "%u", &ping_data_size))
+            if (sscanf(optparse_state.optarg, "%u", &ping_data_size) != 1)
                 usage(1);
-
+            size_flag = 1;
             break;
 
         case 'h':
@@ -670,13 +719,21 @@ int main(int argc, char **argv)
         case 'Q':
             verbose_flag = 0;
             quiet_flag = 1;
-            if (!sscanf(optparse_state.optarg, "%f", &opt_value_float))
+            if (sscanf(optparse_state.optarg, "%f", &opt_value_float) != 1)
                 usage(1);
             if (opt_value_float < 0) {
                 usage(1);
             }
             report_interval = opt_value_float * 1e9;
 
+            /* recognize keyword(s) after number, ignore everything else */
+            {
+                char *comma = strchr(optparse_state.optarg, ',');
+                if ((comma != NULL) && (strcmp(++comma, "cumulative") == 0)) {
+                    cumulative_stats_flag = 1;
+                }
+            }
+
             break;
 
         case 'e':
@@ -780,12 +837,12 @@ int main(int argc, char **argv)
                 usage(1);
 
             if (socket4 >= 0)
-                if(-1 == setsockopt(socket4, SOL_SOCKET, SO_MARK, &fwmark, sizeof fwmark))
+                if(-1 == p_setsockopt(suid, socket4, SOL_SOCKET, SO_MARK, &fwmark, sizeof fwmark))
                     perror("fwmark ipv4");
 
 #ifdef IPV6
             if (socket6 >= 0)
-                if(-1 == setsockopt(socket6, SOL_SOCKET, SO_MARK, &fwmark, sizeof fwmark))
+                if(-1 == p_setsockopt(suid, socket6, SOL_SOCKET, SO_MARK, &fwmark, sizeof fwmark))
                     perror("fwmark ipv6");
 #endif
 
@@ -840,7 +897,7 @@ int main(int argc, char **argv)
             break;
 
         case 'O':
-            if (sscanf(optparse_state.optarg, "%i", &tos)) {
+            if (sscanf(optparse_state.optarg, "%i", &tos) == 1) {
                 if (socket4 >= 0) {
                     if (setsockopt(socket4, IPPROTO_IP, IP_TOS, &tos, sizeof(tos))) {
                         perror("setting type of service octet IP_TOS");
@@ -916,7 +973,7 @@ int main(int argc, char **argv)
 #endif
 
     if (ping_data_size > MAX_PING_DATA) {
-        fprintf(stderr, "%s: data size %u not valid, must be lower than %u\n",
+        fprintf(stderr, "%s: data size %u not valid, must not be larger than %u\n",
             prog, ping_data_size, (unsigned int)MAX_PING_DATA);
         exit(1);
     }
@@ -927,6 +984,11 @@ int main(int argc, char **argv)
         exit(1);
     }
 
+    if (icmp_request_typ == 13 && size_flag != 0) {
+        fprintf(stderr, "%s: cannot change ICMP Timestamp size\n", prog);
+        exit(1);
+    }
+
     if (count_flag) {
         if (verbose_flag)
             per_recv_flag = 1;
@@ -1257,6 +1319,7 @@ void add_cidr(char *addr)
         exit(1);
     }
     net_addr = ntohl(((struct sockaddr_in *)addr_res->ai_addr)->sin_addr.s_addr);
+    freeaddrinfo(addr_res);
 
     /* check mask */
     if (mask < 1 || mask > 32) {
@@ -1278,15 +1341,7 @@ void add_cidr(char *addr)
     }
 
     /* add all hosts in that network (net_addr and net_last inclusive) */
-    for (; net_addr <= net_last; net_addr++) {
-        struct in_addr in_addr_tmp;
-        char buffer[20];
-        in_addr_tmp.s_addr = htonl(net_addr);
-        inet_ntop(AF_INET, &in_addr_tmp, buffer, sizeof(buffer));
-        add_name(buffer);
-    }
-
-    freeaddrinfo(addr_res);
+    add_addr_range_ipv4(net_addr, net_last);
 }
 
 void add_range(char *start, char *end)
@@ -1330,7 +1385,14 @@ void add_range(char *start, char *end)
     end_long = ntohl(((struct sockaddr_in *)addr_res->ai_addr)->sin_addr.s_addr);
     freeaddrinfo(addr_res);
 
-    if (end_long > start_long + MAX_GENERATE) {
+    /* add IPv4 addresses from closed interval [start_long,end_long] */
+    add_addr_range_ipv4(start_long, end_long);
+}
+
+void add_addr_range_ipv4(unsigned long start_long, unsigned long end_long)
+{
+    /* check if generator limit is exceeded */
+    if (end_long >= start_long + MAX_GENERATE) {
         fprintf(stderr, "%s: -g parameter generates too many addresses\n", prog);
         exit(1);
     }
@@ -1366,7 +1428,7 @@ void main_loop()
 
             if (per_recv_flag) {
                 if (timestamp_flag) {
-                    printf("[%.5f] ", (double)current_time_ns / 1e9);
+                    print_timestamp_format(current_time_ns, timestamp_format_flag);
                 }
                 printf("%-*s : [%d], timed out",
                     max_hostname_len, h->host, event->ping_index);
@@ -1813,7 +1875,9 @@ void print_per_system_splits(void)
         }
 
         fprintf(stderr, "\n");
-        stats_reset_interval(h);
+        if (!cumulative_stats_flag) {
+            stats_reset_interval(h);
+        }
     }
 }
 
@@ -1883,6 +1947,7 @@ int send_ping(HOST_ENTRY *h, int index)
     int n;
     int myseq;
     int ret = 1;
+    uint8_t proto = ICMP_ECHO;
 
     update_current_time();
     h->last_send_time = current_time_ns;
@@ -1891,7 +1956,9 @@ int send_ping(HOST_ENTRY *h, int index)
     dbg_printf("%s [%d]: send ping\n", h->host, index);
 
     if (h->saddr.ss_family == AF_INET && socket4 >= 0) {
-        n = socket_sendto_ping_ipv4(socket4, (struct sockaddr *)&h->saddr, h->saddr_len, myseq, ident4);
+        if(icmp_request_typ == 13)
+            proto = ICMP_TSTAMP;
+        n = socket_sendto_ping_ipv4(socket4, (struct sockaddr *)&h->saddr, h->saddr_len, myseq, ident4, proto);
     }
 #ifdef IPV6
     else if (h->saddr.ss_family == AF_INET6 && socket6 >= 0) {
@@ -1986,7 +2053,11 @@ select_again:
 }
 
 int receive_packet(int64_t wait_time,
+#if HAVE_SO_TIMESTAMPNS
     int64_t *reply_timestamp,
+#else
+    int64_t *reply_timestamp __attribute__((unused)),
+#endif
     struct sockaddr *reply_src_addr,
     size_t reply_src_addr_len,
     char *reply_buf,
@@ -2125,13 +2196,20 @@ int decode_icmp_ipv4(
     char *reply_buf,
     size_t reply_buf_len,
     unsigned short *id,
-    unsigned short *seq)
+    unsigned short *seq,
+    int *ip_header_tos,
+    int *ip_header_ttl,
+    uint32_t *ip_header_otime_ms,
+    uint32_t *ip_header_rtime_ms,
+    uint32_t *ip_header_ttime_ms)
 {
     struct icmp *icp;
     int hlen = 0;
 
     if (!using_sock_dgram4) {
         struct ip *ip = (struct ip *)reply_buf;
+        *ip_header_tos = ip->ip_tos;
+        *ip_header_ttl = ip->ip_ttl;
 
 #if defined(__alpha__) && __STDC__ && !defined(__GLIBC__) && !defined(__NetBSD__) && !defined(__OpenBSD__)
         /* The alpha headers are decidedly broken.
@@ -2156,7 +2234,8 @@ int decode_icmp_ipv4(
 
     icp = (struct icmp *)(reply_buf + hlen);
 
-    if (icp->icmp_type != ICMP_ECHOREPLY) {
+    if ((icmp_request_typ == 0 && icp->icmp_type != ICMP_ECHOREPLY) ||
+        (icmp_request_typ == 13 && icp->icmp_type != ICMP_TSTAMPREPLY)) {
         /* Handle other ICMP packets */
         struct icmp *sent_icmp;
         SEQMAP_VALUE *seqmap_value;
@@ -2171,7 +2250,9 @@ int decode_icmp_ipv4(
 
         sent_icmp = (struct icmp *)(reply_buf + hlen + ICMP_MINLEN + sizeof(struct ip));
 
-        if (sent_icmp->icmp_type != ICMP_ECHO || sent_icmp->icmp_id != ident4) {
+        if ((icmp_request_typ == 0 && sent_icmp->icmp_type != ICMP_ECHO) ||
+            (icmp_request_typ == 13 && sent_icmp->icmp_type != ICMP_TSTAMP) ||
+            sent_icmp->icmp_id != ident4) {
             /* not caused by us */
             return -1;
         }
@@ -2222,6 +2303,22 @@ int decode_icmp_ipv4(
 
     *id = icp->icmp_id;
     *seq = ntohs(icp->icmp_seq);
+    if(icp->icmp_type == ICMP_TSTAMPREPLY) {
+
+        /* Check that reply_buf_len is sufficiently big to contain the timestamps */
+        if (reply_buf_len < hlen + ICMP_MINLEN + ICMP_TIMESTAMP_DATA_SIZE) {
+            if (verbose_flag) {
+                char buf[INET6_ADDRSTRLEN];
+                getnameinfo(response_addr, response_addr_len, buf, INET6_ADDRSTRLEN, NULL, 0, NI_NUMERICHOST);
+                printf("received packet too short for ICMP Timestamp Reply (%d bytes from %s)\n", (int)reply_buf_len, buf);
+            }
+            return -1;
+        }
+
+        *ip_header_otime_ms = ntohl(icp->icmp_dun.id_ts.its_otime);
+        *ip_header_rtime_ms = ntohl(icp->icmp_dun.id_ts.its_rtime);
+        *ip_header_ttime_ms = ntohl(icp->icmp_dun.id_ts.its_ttime);
+    }
 
     return hlen;
 }
@@ -2332,6 +2429,12 @@ int wait_for_reply(int64_t wait_time)
     SEQMAP_VALUE *seqmap_value;
     unsigned short id;
     unsigned short seq;
+    int ip_header_tos = -1;
+    int ip_header_ttl = -1;
+    // ICMP Timestamp
+    uint32_t ip_header_otime_ms = 0x80000000U;
+    uint32_t ip_header_rtime_ms = 0x80000000U;
+    uint32_t ip_header_ttime_ms = 0x80000000U;
 
     /* Receive packet */
     result = receive_packet(wait_time, /* max. wait time, in ns */
@@ -2358,7 +2461,12 @@ int wait_for_reply(int64_t wait_time)
             buffer,
             sizeof(buffer),
             &id,
-            &seq);
+            &seq,
+            &ip_header_tos,
+            &ip_header_ttl,
+            &ip_header_otime_ms,
+            &ip_header_rtime_ms,
+            &ip_header_ttime_ms);
         if (ip_hlen < 0) {
             return 1;
         }
@@ -2408,6 +2516,12 @@ int wait_for_reply(int64_t wait_time)
 
     dbg_printf("received [%d] from %s\n", this_count, h->host);
 
+    /* optionally require reply source equal to target address */
+    if (check_source_flag && addr_cmp((struct sockaddr *)&response_addr, (struct sockaddr *)&h->saddr)) {
+        dbg_printf("discarding reply from wrong source address\n");
+        return 1;
+    }
+
     /* discard duplicates */
     if (!loop_flag && h->resp_times[this_count] >= 0) {
         if (!per_recv_flag) {
@@ -2460,24 +2574,13 @@ int wait_for_reply(int64_t wait_time)
 
             if (verbose_flag)
                 printf(" is alive");
-
-            if (elapsed_flag)
-                printf(" (%s ms)", sprint_tm(this_reply));
-
-            if (addr_cmp((struct sockaddr *)&response_addr, (struct sockaddr *)&h->saddr)) {
-                char buf[INET6_ADDRSTRLEN];
-                getnameinfo((struct sockaddr *)&response_addr, sizeof(response_addr), buf, INET6_ADDRSTRLEN, NULL, 0, NI_NUMERICHOST);
-                fprintf(stderr, " [<- %s]", buf);
-            }
-
-            printf("\n");
         }
     }
 
     /* print received ping (unless --quiet) */
     if (per_recv_flag) {
         if (timestamp_flag) {
-            printf("[%.5f] ", (double)recv_time / 1e9);
+            print_timestamp_format(recv_time, timestamp_format_flag);
         }
         avg = h->total_time / h->num_recv;
         printf("%-*s : [%d], %d bytes, %s ms",
@@ -2492,6 +2595,9 @@ int wait_for_reply(int64_t wait_time)
             printf("%d%% return)",
                 (h->num_recv_total * 100) / h->num_sent);
         }
+    }
+
+    if (verbose_flag || alive_flag || per_recv_flag) {
 
         if (addr_cmp((struct sockaddr *)&response_addr, (struct sockaddr *)&h->saddr)) {
             char buf[INET6_ADDRSTRLEN];
@@ -2499,6 +2605,34 @@ int wait_for_reply(int64_t wait_time)
             fprintf(stderr, " [<- %s]", buf);
         }
 
+        if (icmp_request_typ == 13) {
+            printf("%s timestamps: Originate=%u Receive=%u Transmit=%u Localreceive=%u",
+                   alive_flag ? "" : ",",
+                   ip_header_otime_ms, ip_header_rtime_ms, ip_header_ttime_ms,
+                   ms_since_midnight_utc(recv_time));
+        }
+
+        if(print_tos_flag) {
+            if(ip_header_tos != -1) {
+                printf(" (TOS %d)", ip_header_tos);
+            }
+            else {
+                printf(" (TOS unknown)");
+            }
+        }
+
+        if (print_ttl_flag) {
+          if(ip_header_ttl != -1) {
+              printf(" (TTL %d)", ip_header_ttl);
+          }
+          else {
+              printf(" (TTL unknown)");
+          }
+        }
+
+        if (elapsed_flag && !per_recv_flag)
+                printf(" (%s ms)", sprint_tm(this_reply));
+
         printf("\n");
     }
 
@@ -2928,6 +3062,61 @@ void ev_remove(struct event_queue *queue, struct event *event)
     event->ev_next = NULL;
 }
 
+/************************************************************
+
+  Function: print_human_readable_time from current_time_ns
+
+*************************************************************/
+void print_timestamp_format(int64_t current_time_ns, int timestamp_format)
+{
+    char time_buffer[100];
+    time_t current_time_s;
+    struct tm *local_time;
+
+    current_time_s = current_time_ns / 1000000000;
+    local_time = localtime(&current_time_s);
+    switch(timestamp_format) {
+        case 1:
+            // timestamp-format ctime
+            strftime(time_buffer, sizeof(time_buffer), "%c", local_time);
+            printf("[%s] ", time_buffer);
+            break;
+        case 2:
+            // timestamp-format iso
+            strftime(time_buffer, sizeof(time_buffer), "%Y-%m-%dT%T%z", local_time);
+            printf("[%s] ", time_buffer);
+            break;
+        case 3:
+            // timestamp-format rfc3339
+            strftime(time_buffer, sizeof(time_buffer), "%Y-%m-%d %H:%M:%S", local_time);
+            printf("[%s] ", time_buffer);
+            break;
+        default:
+            printf("[%.5f] ", (double)current_time_ns / 1e9);
+    }
+}
+
+/************************************************************
+
+  Function: ms_since_midnight_utc
+
+*************************************************************
+
+  Input: int64_t: current UTC time in ns
+
+  Output: uint32_t: current time in ms since midnight UTC
+
+  Description:
+
+  Return ICMP Timestamp value corresponding to the given time value.
+  The given time value must be in UTC.
+
+*************************************************************/
+static uint32_t ms_since_midnight_utc(int64_t time_val)
+{
+    return (uint32_t)((time_val / 1000000) % (24 * 60 * 60 * 1000));
+}
+
 /************************************************************
 
   Function: usage
@@ -2952,7 +3141,8 @@ void usage(int is_error)
     fprintf(out, "   -B, --backoff=N    set exponential backoff factor to N (default: 1.5)\n");
     fprintf(out, "   -c, --count=N      count mode: send N pings to each target and report stats\n");
     fprintf(out, "   -f, --file=FILE    read list of targets from a file ( - means stdin)\n");
-    fprintf(out, "   -g, --generate     generate target list (only if no -f specified)\n");
+    fprintf(out, "   -g, --generate     generate target list (only if no -f specified),\n");
+    fprintf(out, "                      limited to at most %d targets\n", MAX_GENERATE);
     fprintf(out, "                      (give start and end IP in the target list, or a CIDR address)\n");
     fprintf(out, "                      (ex. %s -g 192.168.1.0 192.168.1.255 or %s -g 192.168.1.0/24)\n", prog, prog);
     fprintf(out, "   -H, --ttl=N        set the IP TTL value (Time To Live hops)\n");
@@ -2974,6 +3164,8 @@ void usage(int is_error)
     fprintf(out, "   -S, --src=IP       set source address\n");
     fprintf(out, "   -t, --timeout=MSEC individual target initial timeout (default: %.0f ms,\n", timeout / 1e6);
     fprintf(out, "                      except with -l/-c/-C, where it's the -p period up to 2000 ms)\n");
+    fprintf(out, "       --check-source discard replies not from target address\n");
+    fprintf(out, "       --icmp-timestamp use ICMP Timestamp instead of ICMP Echo\n");
     fprintf(out, "\n");
     fprintf(out, "Output options:\n");
     fprintf(out, "   -a, --alive        show targets that are alive\n");
@@ -2981,16 +3173,20 @@ void usage(int is_error)
     fprintf(out, "   -C, --vcount=N     same as -c, report results (not stats) in verbose format\n");
     fprintf(out, "   -d, --rdns         show targets by name (force reverse-DNS lookup)\n");
     fprintf(out, "   -D, --timestamp    print timestamp before each output line\n");
+    fprintf(out, "       --timestamp-format=FORMAT  show timestamp in the given format (-D required): ctime|iso|rfc3339\n");
     fprintf(out, "   -e, --elapsed      show elapsed time on return packets\n");
     fprintf(out, "   -n, --name         show targets by name (reverse-DNS lookup for target IPs)\n");
     fprintf(out, "   -N, --netdata      output compatible for netdata (-l -Q are required)\n");
     fprintf(out, "   -o, --outage       show the accumulated outage time (lost packets * packet interval)\n");
     fprintf(out, "   -q, --quiet        quiet (don't show per-target/per-ping results)\n");
-    fprintf(out, "   -Q, --squiet=SECS  same as -q, but add interval summary every SECS seconds\n");
+    fprintf(out, "   -Q, --squiet=SECS[,cumulative]  same as -q, but add interval summary every SECS seconds,\n");
+    fprintf(out, "                                   with 'cumulative', print stats since beginning\n");
     fprintf(out, "   -s, --stats        print final stats\n");
     fprintf(out, "   -u, --unreach      show targets that are unreachable\n");
     fprintf(out, "   -v, --version      show version\n");
     fprintf(out, "   -x, --reachable=N  shows if >=N hosts are reachable or not\n");
     fprintf(out, "   -X, --fast-reachable=N exits true immediately when N hosts are found\n");
+    fprintf(out, "       --print-tos    show received TOS value\n");
+    fprintf(out, "       --print-ttl    show IP TTL value\n");
     exit(is_error);
 }
